#include <Arduino.h>
#include <BMP280_DEV.h>
#include <MPU9250.h>
#include "ultrasonic.hpp"

#define LOOP_INTERVAL_US (10 * 1000)    // 10 * 1000us
#define   US_INTERVAL_US (400 * 29)     // 400cm * 29us/cm

MPU9250 mpu;

void toggleLed() {
  digitalWrite(LED_BUILTIN, !digitalRead(LED_BUILTIN));
}

void setup() {
  Serial.begin(1000000);
  Serial.println(F("boot"));
  pinMode(LED_BUILTIN, OUTPUT);
  
  //initialize acustic sensor
  us_init();

  //initialize I2C MPU
  Wire.begin();
  delay(2000);

  Serial.println(F("setup mpu"));
  mpu.setup(0x68); 

  delay(5000);

  Serial.println(F("calibrate accel/gyro"));
  mpu.calibrateAccelGyro();

  Serial.println(F("calibrate mag"));
  mpu.calibrateMag();
  Serial.println(F("FIELDS:\tUS_0\tUS_1\tMAG_X\tMAG_Y\tMAG_Z\tACCEL_X\tACCEL_Y\tACCEL_Z\tGYRO_X\tGYRO_Y\tGYRO_Z\tTEMP\tEXEC_TIME"));

}

unsigned long loopStart = 0, usTransmitStart = 0;
void loop() {
  if(micros() >= usTransmitStart + US_INTERVAL_US) {
    usTransmitStart = micros();
    //emit ultrasound pulse
    us_transmit();
    toggleLed();
  }

  if(micros() >= loopStart + LOOP_INTERVAL_US) {
    loopStart = micros();

    //get mpu values
    mpu.update();

    Serial.print(F("DATA:\t"));

    //acustic travel time in microseconds 
    for(byte i=0; i<2; i++) {
      Serial.print(us_get_duration(i));
      Serial.print("\t");
    }

    //magnetic field
    for(byte i=0; i<2; i++) {
      Serial.print(mpu.getMag(i));
      Serial.print("\t");
    }

    //accelerometer
    for(byte i=0; i<2; i++) {
      Serial.print(mpu.getAcc(i));
      Serial.print("\t");
    }

    //gyroscope
    for(byte i=0; i<2; i++) {
      Serial.print(mpu.getGyro(i));
      Serial.print("\t");
    }

    //temperature
    Serial.print(mpu.getTemperature());
    Serial.print("\t");

    //execution time in microseconds
    Serial.print(micros() - loopStart);
    Serial.println();

  }

}